'use strict';

var _createAction = require('./createAction');

var _createAction2 = _interopRequireDefault(_createAction);

var _actionwareReducer = require('./actionwareReducer');

var _stateKeeper = require('./stateKeeper');

var _constants = require('./constants');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

describe('ActionwareReducer', function () {
  var wrappedAction = function wrappedAction() {};
  var trackedAction = (0, _createAction2.default)(wrappedAction);

  it('should return previous state if no trackedAction is provided', function () {
    var previousState = {};
    var newState = (0, _actionwareReducer.actionwareReducer)(previousState, { type: 'lorem', payload: 1 });
    expect(newState).to.equal(previousState);
  });

  it('should save new state', function () {
    var action = { type: 'success', _trackedAction: trackedAction };
    (0, _actionwareReducer.actionwareReducer)({}, action);

    expect((0, _stateKeeper.getError)(wrappedAction)).to.equal(null);
    expect((0, _stateKeeper.isBusy)(wrappedAction)).to.equal(false);
  });

  describe('on error event', function () {
    var action = { _actionwareType: _constants.ERROR_TYPE, payload: new Error('err'), _trackedAction: trackedAction };
    var newState = (0, _actionwareReducer.actionwareReducer)({}, action);

    it('should set error state to the given payload', function () {
      expect(newState[trackedAction._errorType]).to.instanceOf(Error);
    });

    it('should set busy state to false', function () {
      expect(newState[trackedAction._busyType]).to.equal(false);
    });
  });

  describe('on busy event', function () {
    var action = { _actionwareType: _constants.BUSY_TYPE, _trackedAction: trackedAction };
    var newState = (0, _actionwareReducer.actionwareReducer)({}, action);

    it('should set error state to null', function () {
      expect(newState[trackedAction._errorType]).to.be.null;
    });

    it('should set busy state to true', function () {
      expect(newState[trackedAction._busyType]).to.equal(true);
    });
  });

  describe('on success event', function () {
    var action = { type: 'success', _trackedAction: trackedAction };
    var newState = (0, _actionwareReducer.actionwareReducer)({}, action);

    it('should set error state to null', function () {
      expect(newState[trackedAction._errorType]).to.be.null;
    });

    it('should set busy state to false', function () {
      expect(newState[trackedAction._busyType]).to.equal(false);
    });
  });
});