'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _call = require('./call');

var _config = require('./config');

describe('call', function () {
  afterEach(function () {
    (0, _call.mockCallWith)(null);
  });

  it('should create tracked action with given function', function () {
    var fn = function fn() {};
    (0, _call.call)(fn);
    expect(_typeof(fn._trackedAction)).to.equal('function');
  });

  it('should invoke given action with given args and store as last arg', function () {
    var store = (0, _config.getStore)();
    var fn = spy();
    (0, _call.call)(fn, 1, 'a', 3);
    expect(fn.getCall(0).args).to.deep.equal([1, 'a', 3, store]);
  });

  it('should use call mock instead of invoking tracked action', function () {
    var callMock = spy();
    (0, _call.mockCallWith)(callMock);
    var fn = spy();
    (0, _call.call)(fn, 1, 'a', 3);

    expect(fn.called).to.be.false;
    expect(callMock.getCall(0).args).to.deep.equal([fn, 1, 'a', 3]);
  });
});