'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _createAction = require('./createAction');

var _createAction2 = _interopRequireDefault(_createAction);

var _config = require('./config');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

describe('createAction', function () {
  it('should throw an error if given action is not a function', function () {
    expect(_createAction2.default.bind(null, 'a')).to.throw;
    expect(_createAction2.default.bind(null, {})).to.throw;
    expect(_createAction2.default.bind(null, 1)).to.throw;
  });

  it('should return a tracked action', function () {
    var action = function action() {};
    var trackedAction = (0, _createAction2.default)(action);
    expect(typeof trackedAction === 'undefined' ? 'undefined' : _typeof(trackedAction)).to.equal('function');
  });

  describe('tracked action', function () {
    it('should be different from given action', function () {
      var action = function action() {};
      var trackedAction = (0, _createAction2.default)(action);
      expect(trackedAction).not.to.equal(action);
    });

    it('should be the same for same given actions', function () {
      var action = function action() {};
      var trackedAction1 = (0, _createAction2.default)(action);
      var trackedAction2 = (0, _createAction2.default)(action);
      expect(trackedAction1).to.equal(trackedAction2);
    });

    it('should have _successType, _errorType and _busyType', function () {
      var action = function action() {};
      var trackedAction = (0, _createAction2.default)(action);
      expect(trackedAction._successType).to.be.ok;
      expect(trackedAction._errorType).to.be.ok;
      expect(trackedAction._busyType).to.be.ok;
    });

    it('should call given action with the same args + store', function () {
      var action = spy();
      var trackedAction = (0, _createAction2.default)(action);
      var store = (0, _config.getStore)();
      trackedAction(1, 'a', 3);
      expect(action.getCall(0).args).to.deep.equal([1, 'a', 3, store]);
    });

    it('should dispatch success event when action executed successfully', function () {
      var action = spy();
      var dispatch = spy();
      var trackedAction = (0, _createAction2.default)(action);
      (0, _config.setup)({ store: { dispatch: dispatch, getState: spy() } });

      trackedAction(1, 'a', 3);

      expect(dispatch.called).to.be.true;
      expect(dispatch.getCall(1).args[0].type).to.equal(trackedAction._successType);
    });

    it('should dispatch busy event before action is executed', function () {
      var action = spy(_asyncToGenerator(regeneratorRuntime.mark(function _callee() {
        return regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                return _context.abrupt('return', Promise.resolve(1));

              case 1:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, undefined);
      })));
      var dispatch = spy();
      var trackedAction = (0, _createAction2.default)(action);

      (0, _config.setup)({ store: { dispatch: dispatch, getState: spy() } });

      trackedAction(1, 'a', 3);

      expect(action.called).to.be.true;
      expect(dispatch.calledBefore(action)).to.be.true;
      expect(dispatch.getCall(0).args[0].type).to.equal(trackedAction._busyType);
    });

    it('should dispatch success event when asynchronous action executed successfully', _asyncToGenerator(regeneratorRuntime.mark(function _callee3() {
      var action, dispatch, trackedAction, result;
      return regeneratorRuntime.wrap(function _callee3$(_context3) {
        while (1) {
          switch (_context3.prev = _context3.next) {
            case 0:
              action = function () {
                var _ref3 = _asyncToGenerator(regeneratorRuntime.mark(function _callee2() {
                  return regeneratorRuntime.wrap(function _callee2$(_context2) {
                    while (1) {
                      switch (_context2.prev = _context2.next) {
                        case 0:
                          return _context2.abrupt('return', Promise.resolve(23));

                        case 1:
                        case 'end':
                          return _context2.stop();
                      }
                    }
                  }, _callee2, undefined);
                }));

                return function action() {
                  return _ref3.apply(this, arguments);
                };
              }();

              dispatch = spy();
              trackedAction = (0, _createAction2.default)(action);


              (0, _config.setup)({ store: { dispatch: dispatch, getState: spy() } });

              _context3.next = 6;
              return trackedAction(1, 'a', 3);

            case 6:
              result = _context3.sent;


              expect(result).to.equal(23);
              expect(dispatch.called).to.be.true;
              expect(dispatch.getCall(1).args[0].type).to.equal(trackedAction._successType);

            case 10:
            case 'end':
              return _context3.stop();
          }
        }
      }, _callee3, undefined);
    })));
  });
});