'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

exports.createReducer = createReducer;

var _createAction = require('./createAction');

var _types = require('./types');

var _constants = require('./constants');

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function addHandler(reducer, handlers, typifier) {
  for (var _len = arguments.length, args = Array(_len > 3 ? _len - 3 : 0), _key = 3; _key < _len; _key++) {
    args[_key - 3] = arguments[_key];
  }

  if (args.length < 2) throw new Error('You should provide at least an action and a handler');

  var handler = args[args.length - 1];
  var types = args.slice(0, args.length - 1);

  if (typeof handler !== 'function') throw new Error('Handler should be a function');

  types.forEach(function (it) {
    switch (typeof it === 'undefined' ? 'undefined' : _typeof(it)) {
      case 'string':
        handlers[it] = handler;
        break;

      case 'function':
        handlers[typifier(it)] = handler;
        break;

      default:
        throw new Error('Invalid action/action type');
    }
  });

  return reducer;
}

function createReducer() {
  var initialState = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  var handlers = {};

  var reducer = function reducer() {
    var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : initialState;
    var _ref = arguments[1];
    var type = _ref.type,
        payload = _ref.payload,
        args = _ref.args;

    if (handlers.hasOwnProperty(type)) {
      var handler = handlers[type];

      if (type.endsWith(_constants.BUSY_TYPE_SUFFIX)) return handler.apply(null, [state].concat(_toConsumableArray(args)));

      if (type.endsWith(_constants.ERROR_TYPE_SUFFIX) || type.endsWith(_constants.CANCELLATION_TYPE_SUFFIX)) return handler.apply(null, [state, payload].concat(_toConsumableArray(args)));

      return handler(state, payload);
    } else {
      return state;
    }
  };

  reducer.on = addHandler.bind(null, reducer, handlers, _createAction.successType);
  reducer.onError = addHandler.bind(null, reducer, handlers, _createAction.errorType);
  reducer.before = addHandler.bind(null, reducer, handlers, _createAction.busyType);

  return reducer;
}