'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _createReducer = require('./createReducer');

var _createAction = require('./createAction');

describe('createReducer', function () {
  it('should return a function', function () {
    var reducer = (0, _createReducer.createReducer)({});
    expect(typeof reducer === 'undefined' ? 'undefined' : _typeof(reducer)).to.equal('function');
  });

  it('returned function should call proper function based on given action type', function () {
    var loadUserByPk = function loadUserByPk() {};
    var loadAllUsers = function loadAllUsers() {};
    var handler = spy();

    var reducer = (0, _createReducer.createReducer)({}).on(loadUserByPk, loadAllUsers, handler);

    reducer({}, { type: (0, _createAction.successType)(loadUserByPk) });
    reducer({}, { type: (0, _createAction.successType)(loadAllUsers) });
    reducer({}, { type: 'loremIpsumDolor' });

    expect(handler.calledTwice).to.equal(true);
  });

  describe('reducer', function () {
    var loadUserByPk = function loadUserByPk() {};

    var reducer = (0, _createReducer.createReducer)({}).on(loadUserByPk, function (state, ignore) {
      return _extends({}, state, { x: 1 });
    }).on('anotherActionName', function (state, ignore) {
      return _extends({}, state, { x: 2 });
    });

    var currentState = { x: 0 };

    it('should return previous state when no type handler is available', function () {
      var newState = reducer(currentState, { type: 'nonExistingActionName' });
      expect(newState).to.equal(currentState);
      expect(newState.x).to.equal(0);
    });

    it('should return a new state according to respective type handler (function)', function () {
      var newState = reducer(currentState, { type: loadUserByPk._trackedAction._successType });
      expect(newState).not.to.equal(currentState);
      expect(newState.x).to.equal(1);
    });

    it('should return a new state according to respective type handler (string)', function () {
      var newState = reducer(currentState, { type: 'anotherActionName' });
      expect(newState).not.to.equal(currentState);
      expect(newState.x).to.equal(2);
    });
  });
});