'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.notifyCancellationListeners = exports.notifyErrorListeners = exports.notifyBeforeListeners = exports.notifySuccessListeners = exports.onCancel = exports.onError = exports.before = exports.onSuccess = undefined;

var _createAction = require('./createAction');

var _config = require('./config');

var globalSuccessListeners = [];
var globalErrorListeners = [];
var globalBeforeListeners = [];
var globalCancellationListeners = [];
var individualListeners = {};

function getListenerAdder(globalListeners, typifier) {
  // Args can be either:
  // 1) action + handler: per action listener
  // 2) handler: global listener
  return function () {

    switch (arguments.length) {
      // global listener
      case 1:
        globalListeners.push(arguments.length <= 0 ? undefined : arguments[0]);
        break;

      // action listener
      case 2:
        var listener = arguments.length <= 1 ? undefined : arguments[1];
        var actionType = typifier(arguments.length <= 0 ? undefined : arguments[0]);

        if (!individualListeners.hasOwnProperty(actionType)) individualListeners[actionType] = [listener];else individualListeners[actionType].push(listener);

        break;

      default:
        // Invalid arguments
        throw new Error('Invalid number of arguments for listener');
    }

    // Make it possible to chain listeners setup
    //
    //  actionware
    //    .onSuccess(...)
    //    .onCancel(...)
    //    .onError(...)
    //    .before(...)
    //
    return {
      onSuccess: onSuccess,
      onCancel: onCancel,
      onError: onError,
      before: before
    };
  };
}

function getListenersNotifier(globalListeners, typifier) {
  return function (_ref) {
    var action = _ref.action,
        payload = _ref.payload,
        args = _ref.args,
        error = _ref.error,
        extras = _ref.extras;

    var store = (0, _config.getStore)();
    var actionType = typifier(action);

    // Notify global listeners
    globalListeners.forEach(function (fn) {
      return fn.call(null, { action: action, payload: payload, args: args, error: error, store: store, extras: extras });
    });

    // Notify individual listeners
    if (individualListeners.hasOwnProperty(actionType)) {
      individualListeners[actionType].forEach(function (fn) {
        return fn.call(null, { action: action, payload: payload, args: args, error: error, store: store, extras: extras });
      });
    }
  };
}

var onSuccess = exports.onSuccess = getListenerAdder(globalSuccessListeners, _createAction.successType);
var before = exports.before = getListenerAdder(globalBeforeListeners, _createAction.busyType);
var onError = exports.onError = getListenerAdder(globalErrorListeners, _createAction.errorType);
var onCancel = exports.onCancel = getListenerAdder(globalCancellationListeners, _createAction.cancellationType);

var notifySuccessListeners = exports.notifySuccessListeners = getListenersNotifier(globalSuccessListeners, _createAction.successType);
var notifyBeforeListeners = exports.notifyBeforeListeners = getListenersNotifier(globalBeforeListeners, _createAction.busyType);
var notifyErrorListeners = exports.notifyErrorListeners = getListenersNotifier(globalErrorListeners, _createAction.errorType);
var notifyCancellationListeners = exports.notifyCancellationListeners = getListenersNotifier(globalCancellationListeners, _createAction.cancellationType);