'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _mochaJsdom = require('mocha-jsdom');

var _mochaJsdom2 = _interopRequireDefault(_mochaJsdom);

var _reactRedux = require('react-redux');

var _enzyme = require('enzyme');

var _withActions = require('./withActions');

var _config = require('./config');

var _call = require('./call');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var store = { dispatch: spy(), getState: spy(), subscribe: spy() };

(0, _config.setup)({ store: store });

describe('withActions', function () {
  (0, _mochaJsdom2.default)();

  var wrapper = void 0;

  var actions = {
    loadUsers: spy(),
    loadUserByPk: spy(),
    login: function login() {
      (0, _call.call)(actions.loadUserByPk, 1, 2);
    }
  };

  var MyComponent = function MyComponent(_ref) {
    var loadUsers = _ref.loadUsers,
        login = _ref.login;
    return _react2.default.createElement(
      'div',
      null,
      _react2.default.createElement(
        'button',
        { className: 'firstButton', onClick: loadUsers },
        'Click here'
      ),
      _react2.default.createElement(
        'button',
        { className: 'secondButton', onClick: login },
        'Click here'
      )
    );
  };

  before(function () {
    var ConnectedComponent = (0, _withActions.withActions)(actions)(MyComponent);

    wrapper = (0, _enzyme.mount)(_react2.default.createElement(
      _reactRedux.Provider,
      { store: store },
      _react2.default.createElement(ConnectedComponent, null)
    ));
  });

  it('should pass actions to wrapped component', function () {
    var mountedProps = wrapper.find(MyComponent).props();
    expect(_typeof(mountedProps.loadUsers)).to.equal('function');
  });

  it('should call action fn with store as last arg', function () {
    var store = (0, _config.getStore)();

    wrapper.find('button.firstButton').simulate('click');
    expect(actions.loadUsers.called).to.equal(true);
    expect(actions.loadUsers.getCall(0).args[1]).to.equal(store);
  });

  it('should call an action from another action with store as last arg', function () {
    var store = (0, _config.getStore)();

    wrapper.find('button.secondButton').simulate('click');
    expect(actions.loadUserByPk.called).to.equal(true);
    expect(actions.loadUserByPk.getCall(0).args[0]).to.equal(1);
    expect(actions.loadUserByPk.getCall(0).args[1]).to.equal(2);
    expect(actions.loadUserByPk.getCall(0).args[2]).to.equal(store);
  });
});